/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.activity;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.GridView;
import android.widget.Toast;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.MetadataItem;
import com.thingspace.files.R;
import com.thingspace.files.adapters.MetadataGridAdapter;
import com.thingspace.files.task.CopyFileFolderTask;
import com.thingspace.files.task.MetadataTask;
import com.thingspace.files.task.MoveFileFolderTask;
import com.thingspace.files.utils.KeplerUtil;

import java.net.HttpURLConnection;

/**
 * Activity to copy/move Files/Folder
 */
public class CopyMoveActivity extends BaseActivity {

    private static final String ROOT_CALL_PATH = "/";
    private static final String MEDIA_TYPE_IMAGE = "image";
    private static final int ACTION_COPY = 0;
    private static final int ACTION_MOVE = 1;
    private static String currentFolderOpened = "";
    private String srcFileFolderPath = "";
    private int copyMoveAction = 0;
    private ProgressDialog dialog;
    private CloudAPI cloudAPI;
    private GridView gridView;
    private MetadataGridAdapter metaAdapter;
    private ItemList<MetadataItem> metadataResponse;
    private String[] fileSizeUnits = {"bytes", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"};

    private AdapterView.OnItemClickListener folderClick = new AdapterView.OnItemClickListener() {
        @Override
        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
            MetadataItem item = metadataResponse.getItem(position);
            if (item.getContentToken() != null) {
                if (item.getMimeType().contains(MEDIA_TYPE_IMAGE)) {

                }
            } else {
                startMetadataTask("/".equals(item.getParentPath()) ? item.getParentPath() + item.getName()
                        : item.getParentPath() + "/" + item.getName());
            }
        }
    };

    /**
     * @param savedInstanceState
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        findViewById(R.id.clickablelayout).setVisibility(View.VISIBLE);
        srcFileFolderPath = getIntent().getStringExtra(FileBrowserMainActivity.EXTRA_COPY_MOVE_SRC);
        copyMoveAction = getIntent().getIntExtra(FileBrowserMainActivity.EXTRA_COPY_MOVE_ACTION, 0);
        cloudAPI = new CloudAPI(getString(R.string.APP_ID), getString(R.string.APP_SECRET), getString(R.string.CALLBACK_URL));
        gridView = (GridView) findViewById(R.id.gridView);
        if (ACTION_MOVE == copyMoveAction) {
            ((Button) findViewById(R.id.move_copy_button)).setText(getString(R.string.move_here));
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (cloudAPI.hasAuthorization(this)) {
            startMetadataTask(ROOT_CALL_PATH);
        }
    }

    /**
     * @param path
     */
    private void startMetadataTask(String path) {
        showBusyDialog(true);
        currentFolderOpened = path;
        MetadataTask task = new MetadataTask(this, path, cloudAPI);
        task.execute();
    }

    /**
     * @param v
     */
    public void onCopyMoveCancelClick(View v) {
        finish();
    }

    /**
     * @param v
     */
    public void onCopyMoveButtonClick(View v) {

        if (!KeplerUtil.isNetworkAvailable(this)) {
            Toast.makeText(this, getString(R.string.error_unavailable), Toast.LENGTH_SHORT).show();
            return;
        }

        if (copyMoveAction == ACTION_COPY) {
            CopyFileFolderTask task = new CopyFileFolderTask(CopyMoveActivity.this, CopyMoveActivity.this,
                    cloudAPI, srcFileFolderPath, currentFolderOpened);
            showBusyDialog(true);
            task.execute();
        } else if (copyMoveAction == ACTION_MOVE) {
            MoveFileFolderTask task = new MoveFileFolderTask(CopyMoveActivity.this, CopyMoveActivity.this,
                    cloudAPI, srcFileFolderPath, currentFolderOpened);
            showBusyDialog(true);
            task.execute();
        }
    }

    /**
     * @param response
     */
    @Override
    public void onMetadataResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false);

        if (responseCode == HttpURLConnection.HTTP_OK) {
            this.metadataResponse = response;

            metaAdapter = new MetadataGridAdapter(this, cloudAPI, R.layout.grid_item, response);
            gridView.setAdapter(metaAdapter);

            gridView.setOnItemClickListener(folderClick);
        } else if (responseCode == HttpURLConnection.HTTP_UNAVAILABLE) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.error_unavailable), Toast.LENGTH_LONG).show();
            return;
        } else if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.error_unathorized), Toast.LENGTH_LONG).show();
            cloudAPI.logout(this);
            Intent intent = new Intent(CopyMoveActivity.this, FileBrowserMainActivity.class);
            intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
            startActivity(intent);
        } else {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.error_happened), Toast.LENGTH_LONG).show();
        }
    }

    @Override
    public void onCopyResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false);
        if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.auth_failed), Toast.LENGTH_LONG).show();
            setResult(responseCode);
            finish();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_OK && response != null) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.toast_copy_message), Toast.LENGTH_LONG).show();
        } else {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.error_happened), Toast.LENGTH_LONG).show();
        }
        setResult(responseCode);
        finish();
    }

    @Override
    public void onMoveResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false);
        if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.auth_failed), Toast.LENGTH_LONG).show();
            setResult(responseCode);
            finish();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_OK && response != null) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.toast_move_message), Toast.LENGTH_LONG).show();
        } else {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.error_happened), Toast.LENGTH_LONG).show();
        }
        setResult(responseCode);
        finish();
    }

    /**
     * @param show
     */
    public void showBusyDialog(boolean show) {
        if (dialog == null && show) {
            dialog = new ProgressDialog(CopyMoveActivity.this);
            dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            dialog.setMessage("Loading. Please wait...");
            dialog.setIndeterminate(true);
            dialog.setCanceledOnTouchOutside(false);
            dialog.show();
        } else if (dialog != null && !show) {
            dialog.dismiss();
            dialog = null;

        }

    }
}