/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.activity;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.GridView;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.MetadataItem;
import com.thingspace.files.R;
import com.thingspace.files.adapters.MetadataGridAdapter;
import com.thingspace.files.task.CreateFolderTask;
import com.thingspace.files.task.DeleteFileFolderTask;
import com.thingspace.files.task.DownloadTask;
import com.thingspace.files.task.GetAccountInfoTask;
import com.thingspace.files.task.MetadataTask;
import com.thingspace.files.task.RenameFileFolderTask;
import com.thingspace.files.task.UploadTask;
import com.thingspace.files.utils.KeplerUtil;
import com.thingspace.files.utils.PathFinder;

import java.net.HttpURLConnection;
import java.text.DecimalFormat;
import java.util.Stack;

import javax.net.ssl.HttpsURLConnection;

/**
 * Activity to display the Files/Folder View
 */
public class FileBrowserMainActivity extends BaseActivity {

    public static final String ACCESS_TOKEN = "accessToken";
    public static final String EXTRA_COPY_MOVE_ACTION = "copy_move_action";
    public static final String EXTRA_COPY_MOVE_SRC = "copy_move_src";
    private static final String ROOT_CALL_PATH = "/";
    private static final String RESPONSE_EXTENSION = "extension";
    private static final String RESPONSE_PARENTPATH = "parentPath";
    private static final String RESPONSE_NAME = "name";
    private static final String RESPONSE_VERSIONCREATED = "versionCreated";
    private static final String RESPONSE_ARTIST = "Artist";
    private static final String RESPONSE_ALBUM = "Album";
    private static final String RESPONSE_GENRE = "Genre";
    private static final String MEDIA_TYPE_AUDIO = "audio";
    private static final String MEDIA_TYPE_IMAGE = "image";
    private static final int ACTION_COPY = 0;
    private static final int ACTION_MOVE = 1;
    private static final int ACTION_DELETE = 2;
    private static final int ACTION_RENAME = 3;
    private static final int ACTION_FILEINFO = 4;
    private static final int ACTION_DOWNLOAD = 5;
    private static final int ONE_MEGA_BYTE = 1024;
    private static final int FILE_SELECT_CODE = 101;
    private static String currentFolderOpened = "";
    private ProgressDialog dialog;
    private CloudAPI cloudAPI;
    private GridView gridView;
    private MetadataGridAdapter metaAdapter;
    private ItemList<MetadataItem> metadataResponse;
    private String[] fileSizeUnits = {"bytes", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"};

    private Stack<String> metaDataPath;

    private AdapterView.OnItemClickListener folderClick = new AdapterView.OnItemClickListener() {
        @Override
        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
            MetadataItem item = metadataResponse.getItem(position);
            if (item == null) {
                return;
            }

            if (item.getContentToken() != null) {
                if (item.getMimeType().contains(MEDIA_TYPE_IMAGE)) {
                    startGetFileTask(item.getParentPath(), item.getName(), item.getSize(), true);
                }
            } else {
                startMetadataTask("/".equals(item.getParentPath()) ? item.getParentPath() + item.getName()
                        : item.getParentPath() + "/" + item.getName(), true);
            }
        }
    };

    private AdapterView.OnItemLongClickListener fileLongClick = new AdapterView.OnItemLongClickListener() {
        @Override
        public boolean onItemLongClick(AdapterView<?> parent, View view, final int position, long id) {
            AlertDialog.Builder fileOperationDialog = new AlertDialog.Builder(FileBrowserMainActivity.this);
            final MetadataItem item = metadataResponse.getItem(position);
            if (item == null || "/".equals(item.getParentPath())) {
                return false;
            }

            final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<String>(
                    FileBrowserMainActivity.this,
                    android.R.layout.simple_list_item_1);
            arrayAdapter.add(getString(R.string.action_copy));
            arrayAdapter.add(getString(R.string.action_move));
            arrayAdapter.add(getString(R.string.action_delete));
            arrayAdapter.add(getString(R.string.action_rename));

            if (item.getContentToken() != null) {
                arrayAdapter.add(getString(R.string.action_file_info));
                arrayAdapter.add(getString(R.string.action_download));
            }

            fileOperationDialog.setAdapter(arrayAdapter, new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int which) {
                    switch (which) {
                        case ACTION_COPY:
                            moveCopyTask(ACTION_COPY, item);
                            break;

                        case ACTION_MOVE:
                            moveCopyTask(ACTION_MOVE, item);
                            break;

                        case ACTION_DELETE:
                            fileInfoAndDeleteTask(ACTION_DELETE, item);
                            break;

                        case ACTION_RENAME:
                            createAndRenameTask(item);
                            break;

                        case ACTION_FILEINFO:
                            fileInfoAndDeleteTask(ACTION_FILEINFO, item);
                            break;
                        case ACTION_DOWNLOAD:
                            downloadFile(item);
                    }
                }
            });

            fileOperationDialog.setTitle(getString(R.string.file_actions));
            fileOperationDialog.create();
            fileOperationDialog.show();

            return true;
        }
    };

    /**
     * @param savedInstanceState
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        metaDataPath = new Stack<>();
        cloudAPI = new CloudAPI(getString(R.string.APP_ID), getString(R.string.APP_SECRET), getString(R.string.CALLBACK_URL));
        gridView = (GridView) findViewById(R.id.gridView);
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (cloudAPI.hasAuthorization(this)) {
            if (currentFolderOpened.isEmpty()) {
                startMetadataTask(ROOT_CALL_PATH, false);
            }
        } else {
            startAuthentication();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        currentFolderOpened = "";
        metaDataPath.clear();
    }

    /**
     * To authorize the user.
     */
    private void startAuthentication() {
        // Start the remote authentication
        cloudAPI.authorize(this);
    }

    /**
     * @param menu
     * @return
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        if ("/".equals(currentFolderOpened)) {
            menu.findItem(R.id.action_create_folder).setVisible(false);
            menu.findItem(R.id.action_upload).setVisible(false);
        }
        return super.onPrepareOptionsMenu(menu);
    }

    /**
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.

        if (!cloudAPI.hasAuthorization(this)) {
            reStartLogin();
            return super.onOptionsItemSelected(item);
        }

        switch (item.getItemId()) {
            case R.id.action_upload:
                showFileSelector();
                break;
            case R.id.action_account_info:
                startAccountTask();
                break;
            case R.id.action_search:
                Intent intent = new Intent(FileBrowserMainActivity.this, SearchActivity.class);
                startActivity(intent);
                break;
            case R.id.action_create_folder:
                createAndRenameTask(null);
                break;
            case R.id.action_refresh:
                startMetadataTask(currentFolderOpened, false);
                break;
            case R.id.action_trash:
                startTrash();
                break;
            case R.id.action_revoke:
                cloudAPI.logout(this);
                finish();
        }
        return super.onOptionsItemSelected(item);
    }

    private void showFileSelector() {
        Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
        intent.setType("*/*");
        intent.addCategory(Intent.CATEGORY_OPENABLE);

        try {
            startActivityForResult(Intent.createChooser(intent, getString(R.string.select_file_upload)), FILE_SELECT_CODE);
        } catch (android.content.ActivityNotFoundException ex) {
            Toast.makeText(this, getString(R.string.install_file_manager), Toast.LENGTH_SHORT).show();
        }
    }

    private void startMetadataTask(String path, boolean addPathToStack) {
        if (cloudAPI.hasAuthorization(this)) {
            showBusyDialog(true, getString(R.string.loading_busy_text));
            if (addPathToStack) {
                metaDataPath.push(currentFolderOpened);
            }
            currentFolderOpened = path;
            MetadataTask task = new MetadataTask(this, path, cloudAPI);
            task.execute();
        } else {
            reStartLogin();
        }
    }

    private void startTrash() {
        Intent intent = new Intent(FileBrowserMainActivity.this, TrashCanActivity.class);
        startActivity(intent);
    }

    private void startGetFileTask(String path, String fileName, long size, boolean canDownload) {
        Intent intent = new Intent(this, PhotoActivity.class);
        intent.putExtra(PhotoActivity.EXTRA_ACCESS_TOKEN, cloudAPI.getAccessToken(this));
        intent.putExtra(PhotoActivity.EXTRA_NAME, fileName);
        intent.putExtra(PhotoActivity.EXTRA_SIZE, size);
        intent.putExtra(PhotoActivity.EXTRA_PARENT_PATH, path);
        intent.putExtra(PhotoActivity.EXTRA_CONTENT_TOKEN, cloudAPI.getAccessToken(this));
        startActivity(intent);
    }

    private void startAccountTask() {
        if (!KeplerUtil.isNetworkAvailable(this)) {
            Toast.makeText(this, getString(R.string.error_unavailable), Toast.LENGTH_SHORT).show();
            return;
        }

        showBusyDialog(true, getString(R.string.loading_busy_text));
        GetAccountInfoTask task = new GetAccountInfoTask(this, cloudAPI, this);
        task.execute();
    }

    /**
     * @param item
     */
    private void createAndRenameTask(final MetadataItem item) {
        AlertDialog.Builder createFolderPopup = new AlertDialog.Builder(FileBrowserMainActivity.this);
        final EditText input = new EditText(FileBrowserMainActivity.this);

        LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.MATCH_PARENT,
                LinearLayout.LayoutParams.MATCH_PARENT);
        input.setLayoutParams(lp);

        if (item != null) {
            input.setText(item.getName());
        }

        createFolderPopup.setView(input);
        createFolderPopup.setTitle(getString(item == null ? R.string.action_create_folder : R.string.action_rename)).setCancelable(false)
                .setMessage(getString(item == null ? R.string.xml_create_folder_text : R.string.rename_dialog_message))
                .setPositiveButton(getString(R.string.action_ok), new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        if (input.getText().toString().isEmpty() || input.getText().toString().trim().isEmpty()) {
                            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.enter_valid_name), Toast.LENGTH_SHORT).show();
                        } else {
                            if (!KeplerUtil.isNetworkAvailable(FileBrowserMainActivity.this)) {
                                Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_unavailable), Toast.LENGTH_SHORT).show();
                                return;
                            }

                            if (item == null) {
                                CreateFolderTask task = new CreateFolderTask(FileBrowserMainActivity.this, FileBrowserMainActivity.this, cloudAPI,
                                        currentFolderOpened, input.getText().toString());
                                showBusyDialog(true, getString(R.string.loading_busy_text));
                                task.execute();
                            } else {
                                String src = item.getParentPath() + "/" + item.getName();
                                String target = item.getParentPath() + "/" + input.getText().toString();
                                if (src.equalsIgnoreCase(target)) {
                                    Toast.makeText(FileBrowserMainActivity.this, getString(R.string.rename_name_same), Toast.LENGTH_SHORT).show();
                                    return;
                                }

                                RenameFileFolderTask task = new RenameFileFolderTask(FileBrowserMainActivity.this, FileBrowserMainActivity.this,
                                        cloudAPI, src, target);
                                showBusyDialog(true, getString(R.string.loading_busy_text));
                                task.execute();
                            }
                        }
                    }
                }).setNegativeButton(getString(R.string.action_cancel), null).create();
        createFolderPopup.show();
    }

    /**
     * @param action
     * @param item
     */
    private void fileInfoAndDeleteTask(final int action, final MetadataItem item) {
        if (item != null) {
            String title = "", dialogMessage = "";
            if (ACTION_DELETE == action) {
                title = getString(R.string.action_delete);
                dialogMessage = getString(R.string.delete_dialog_message);
            } else if (ACTION_FILEINFO == action) {
                dialogMessage = getString(R.string.delete_dialog_message);
                title = getString(R.string.action_file_info);
                dialogMessage = RESPONSE_NAME + " : " + item.getName()
                        + "\n" + RESPONSE_PARENTPATH + " : " + item.getParentPath()
                        + "\n" + RESPONSE_EXTENSION + " : " + item.getExtension()
                        + "\n" + RESPONSE_VERSIONCREATED + " : " + item.getCreationDate();

                if (item.getMimeType().contains(MEDIA_TYPE_AUDIO)) {
                    dialogMessage = dialogMessage + "\n"
                            + RESPONSE_ALBUM + " : " + item.getAlbum()
                            + "\n" + RESPONSE_GENRE + " : " + item.getGenre()
                            + "\n" + RESPONSE_ARTIST + " : " + item.getArtist();
                }
            }

            AlertDialog.Builder deleteDialog = new AlertDialog.Builder(FileBrowserMainActivity.this)
                    .setTitle(title).setCancelable(false)
                    .setMessage(dialogMessage).setPositiveButton(ACTION_DELETE == action ? getString(R.string.action_yes)
                                    : getString(R.string.action_ok),
                            new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    if (ACTION_DELETE == action) {
                                        if (!KeplerUtil.isNetworkAvailable(FileBrowserMainActivity.this)) {
                                            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_unavailable), Toast.LENGTH_SHORT).show();
                                            return;
                                        }

                                        showBusyDialog(true, getString(R.string.loading_busy_text));
                                        DeleteFileFolderTask task = new DeleteFileFolderTask(FileBrowserMainActivity.this, FileBrowserMainActivity.this,
                                                cloudAPI, item.getParentPath() + "/" + item.getName());
                                        task.execute();
                                    }
                                }
                            });
            if (ACTION_DELETE == action) {
                deleteDialog.setNegativeButton(getString(R.string.action_no), null);
            }
            deleteDialog.create();
            deleteDialog.show();
        }
    }

    /**
     * @param action
     * @param item
     */
    private void moveCopyTask(final int action, final MetadataItem item) {
        Intent intent = new Intent(FileBrowserMainActivity.this, CopyMoveActivity.class);
        intent.putExtra(EXTRA_COPY_MOVE_ACTION, action);
        intent.putExtra(ACCESS_TOKEN, cloudAPI.getAccessToken(this));
        intent.putExtra(EXTRA_COPY_MOVE_SRC, item.getParentPath()
                + "/" + item.getName());
        startActivityForResult(intent, action);
    }

    private void downloadFile(MetadataItem item) {
        DownloadTask downloadTask = new DownloadTask(this, item.getParentPath(), item.getName(), cloudAPI);
        downloadTask.execute();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case ACTION_MOVE:
                if (resultCode == HttpURLConnection.HTTP_OK) {
                    startMetadataTask(currentFolderOpened, false);
                } else if (resultCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
                    reStartLogin();
                }
                break;
            case FILE_SELECT_CODE:
                if (resultCode == RESULT_OK) {
                    // Get the Uri of the selected file
                    Uri uri = data.getData();
                    // Get the path
                    String path = PathFinder.getPath(this, uri);
                    if (TextUtils.isEmpty(path)) {
                        Toast.makeText(this, R.string.error_selecting_file, Toast.LENGTH_SHORT).show();
                        break;
                    }
                    String parentPath = path.substring(0, path.lastIndexOf("/"));
                    String fileName = path.substring(path.lastIndexOf("/") + 1);
                    // Initiate the upload
                    showBusyDialog(true, getString(R.string.uploading_busy_text));
                    new UploadTask(this, currentFolderOpened, fileName, parentPath, cloudAPI).execute();
                }
                break;
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    /**
     * @param response
     */
    @Override
    public void onMetadataResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false, null);
        this.invalidateOptionsMenu();
        
        if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            if (!metaDataPath.empty()) {
                metaDataPath.pop();
                currentFolderOpened = !metaDataPath.empty() ? metaDataPath.peek() : ROOT_CALL_PATH;
            }
            reStartLogin();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_UNAVAILABLE) {
            if (!metaDataPath.empty()) {
                metaDataPath.pop();
                currentFolderOpened = !metaDataPath.empty() ? metaDataPath.peek() : ROOT_CALL_PATH;
            }
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_unavailable), Toast.LENGTH_SHORT).show();
            return;
        }

        if (response == null || responseCode != HttpURLConnection.HTTP_OK) {
            if (!metaDataPath.empty()) {
                metaDataPath.pop();
                currentFolderOpened = !metaDataPath.empty() ? metaDataPath.peek() : ROOT_CALL_PATH;
            }
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_happened), Toast.LENGTH_SHORT).show();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_OK) {
            this.metadataResponse = response;
            metaAdapter = new MetadataGridAdapter(this, cloudAPI, R.layout.grid_item, response);
            gridView.setAdapter(metaAdapter);

            gridView.setOnItemClickListener(folderClick);
            gridView.setOnItemLongClickListener(fileLongClick);
        }
    }

    @Override
    public void onGetFileResponseReceived(String response, int responseCode) {
        showBusyDialog(false, null);
        if (responseCode == HttpURLConnection.HTTP_OK) {
            Toast.makeText(FileBrowserMainActivity.this, response, Toast.LENGTH_SHORT).show();
        } else if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            reStartLogin();
        } else {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_happened), Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * @param availableStorage
     * @param usedStorage
     */
    @Override
    public void onAccountInfoResponseReceived(int responseCode, Long availableStorage, Long usedStorage) {
        showBusyDialog(false, null);
        if (responseCode == HttpURLConnection.HTTP_OK) {
            String alertMessage = "Used " + convertToFormatSize(usedStorage.doubleValue())
                    + " of " + convertToFormatSize(availableStorage.doubleValue());
            AlertDialog storagePopup = new AlertDialog.Builder(FileBrowserMainActivity.this)
                    .setTitle(getString(R.string.action_account_info)).setCancelable(false).setMessage(alertMessage)
                    .setPositiveButton(getString(R.string.action_ok), null).create();
            storagePopup.show();
        } else if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            reStartLogin();
        } else if (responseCode == HttpURLConnection.HTTP_UNAVAILABLE) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_unavailable), Toast.LENGTH_SHORT).show();
        } else {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_happened), Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * @param metadataItem
     * @param responseCode
     */
    @Override
    public void onCreateFolderResponseReceived(MetadataItem metadataItem, int responseCode) {
        showBusyDialog(false, null);
        if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            reStartLogin();
            return;
        }

        if ((responseCode == HttpsURLConnection.HTTP_OK)) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.folder_already_created), Toast.LENGTH_SHORT).show();
            return;
        }

        if (metadataItem == null || responseCode != HttpsURLConnection.HTTP_CREATED) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_happened), Toast.LENGTH_SHORT).show();
            return;
        }
        if (responseCode == HttpsURLConnection.HTTP_CREATED && metadataItem != null) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.folder_created), Toast.LENGTH_SHORT).show();
            startMetadataTask(currentFolderOpened, false);
        }

    }

    @Override
    public void onDeleteResponseReceived(boolean response) {
        showBusyDialog(false, null);
        if (!response) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_happened), Toast.LENGTH_SHORT).show();
            return;
        }

        if (response) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.delete_completed), Toast.LENGTH_SHORT).show();
            startMetadataTask(currentFolderOpened, false);
        }
    }

    @Override
    public void onRenameResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false, null);
        if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            reStartLogin();
            return;
        }

        if (response == null || responseCode != HttpURLConnection.HTTP_OK) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_happened), Toast.LENGTH_SHORT).show();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_OK && response != null) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.toast_rename_message), Toast.LENGTH_SHORT).show();
            startMetadataTask(currentFolderOpened, false);
        }
    }

    @Override
    public void onCopyResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false, null);

        if (response == null) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_happened), Toast.LENGTH_SHORT).show();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_OK) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.toast_copy_message), Toast.LENGTH_SHORT).show();
        }
    }

    @Override
    public void onMoveResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false, null);

        if (response == null) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_happened), Toast.LENGTH_SHORT).show();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_OK) {
            Toast.makeText(FileBrowserMainActivity.this, getString(R.string.toast_move_message), Toast.LENGTH_SHORT).show();
            startMetadataTask(currentFolderOpened, false);
        }
    }

    /**
     * @param bytes
     * @return
     */
    public String convertToFormatSize(double bytes) {
        String sizeToReturn = "";
        int index = 0;
        for (index = 0; index < fileSizeUnits.length; index++) {
            if (bytes < ONE_MEGA_BYTE) {
                break;
            }
            bytes = bytes / ONE_MEGA_BYTE;
        }
        DecimalFormat df = new DecimalFormat("#.##");
        sizeToReturn = String.valueOf(df.format(bytes)) + " " + fileSizeUnits[index];
        return sizeToReturn;
    }

    /**
     * @param show
     */
    public void showBusyDialog(boolean show, String message) {
        if (dialog == null && show) {
            dialog = new ProgressDialog(FileBrowserMainActivity.this);
            dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            dialog.setMessage(message);
            dialog.setIndeterminate(true);
            dialog.setCanceledOnTouchOutside(false);
            dialog.show();
        } else if (dialog != null && !show) {
            dialog.dismiss();
            dialog = null;

        }
    }

    /**
     * Called to Re-Launch the app when there is a invalid/epired token
     */
    private void reStartLogin() {
        Toast.makeText(FileBrowserMainActivity.this, getString(R.string.error_unathorized), Toast.LENGTH_SHORT).show();
        cloudAPI.logout(this);
        Intent intent = new Intent(FileBrowserMainActivity.this, FileBrowserMainActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivity(intent);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK && metaDataPath.size() >= 1) {
            startMetadataTask(metaDataPath.pop(), false);
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public void onUploadDownloadResponseReceived(boolean response, int responseCode) {
        showBusyDialog(false, null);
        if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            reStartLogin();
            return;
        } else if (responseCode == HttpURLConnection.HTTP_UNAVAILABLE) {
            Toast.makeText(this, getString(R.string.error_unavailable), Toast.LENGTH_SHORT).show();
        } else if (response) {
            startMetadataTask(currentFolderOpened, false);
        }
    }

}