/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.activity;

import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.thingspace.files.R;
import com.thingspace.files.cache.ImageCache;
import com.thingspace.files.cache.ImageFetcher;
import com.thingspace.files.task.GetFileTask;
import com.thingspace.files.utils.KeplerUtil;

import java.io.File;
import java.io.FileOutputStream;

/**
 * Activity to display full screen photo
 */
public class PhotoActivity extends AppCompatActivity {

    public static final String EXTRA_NAME = "extra_name";
    public static final String EXTRA_PARENT_PATH = "extra_parent_path";
    public static final String EXTRA_ACCESS_TOKEN = "extra_access_token";
    public static final String EXTRA_MIME_TYPE = "mime-type";
    public static final String EXTRA_CAPTURED_DATE = "capturedDate";
    public static final String EXTRA_WIDTH = "width";
    public static final String EXTRA_CREATED_DATE = "createdDate";
    public static final String EXTRA_HEIGHT = "height";
    public static final String EXTRA_SIZE = "size";
    public static final String EXTRA_CONTENT_TOKEN = "content_token";

    private static ImageFetcher staticImageFetcher;

    private ImageView fullPhoto;
    private ProgressBar progressBar;
    private String name;
    private String parentPath;
    private String mimeType;
    private String capturedDate;
    private String width;
    private String height;
    private Long size;
    private String contentToken;
    private String accessToken;

    private GetFileTask getFileTask;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_photo);
        fullPhoto = (ImageView) findViewById(R.id.imageFullPhoto);

        progressBar = (ProgressBar) findViewById(R.id.progressBarPhoto);
        accessToken = getIntent().getStringExtra(EXTRA_ACCESS_TOKEN);

        mimeType = getIntent().getStringExtra(EXTRA_MIME_TYPE);
        capturedDate = getIntent().getStringExtra(EXTRA_CAPTURED_DATE);
        width = getIntent().getStringExtra(EXTRA_WIDTH);
        height = getIntent().getStringExtra(EXTRA_HEIGHT);
        size = getIntent().getLongExtra(EXTRA_SIZE, 0L);
        name = getIntent().getStringExtra(EXTRA_NAME);
        parentPath = getIntent().getStringExtra(EXTRA_PARENT_PATH);
        contentToken = getIntent().getStringExtra(EXTRA_CONTENT_TOKEN);

        loadFullPhoto();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_photo, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        switch (item.getItemId()) {
            case R.id.action_download:
                // Download the photo
                saveImageToSD();
                return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void loadFullPhoto() {
        if (!KeplerUtil.isNetworkAvailable(this)) {
            Toast.makeText(this, getString(R.string.error_unavailable), Toast.LENGTH_SHORT).show();
            finish();
            return;
        }

        String url = getString(R.string.THINGSPACE_SERVER_URL) + "/cloud/1/thumbnails/" + contentToken + "?th=200&tw=200";
        getImageFetcher().loadImageFromCache(url, fullPhoto);
        progressBar.setVisibility(View.VISIBLE);
        if (getFileTask != null && !getFileTask.getStatus().equals(AsyncTask.Status.FINISHED)) {
            getFileTask.cancel(true);
        }
        getFileTask = new GetFileTask(parentPath + "/" + name, accessToken, fullPhoto, this, progressBar);
        getFileTask.execute();
    }

    /**
     * Retrieve the photo bytes from imageview bitmap and write it to the storage.
     */
    private void saveImageToSD() {
        File sdcard = Environment.getExternalStorageDirectory();

        File folder = new File(sdcard.getAbsoluteFile(), "VZDemoApp");//the dot makes this directory hidden to the user
        folder.mkdir();
        File file = new File(folder.getAbsoluteFile(), name);
        if (file.exists()) {
            Toast.makeText(this, getString(R.string.download_exist), Toast.LENGTH_SHORT).show();
            return;
        }

        try {
            FileOutputStream out = new FileOutputStream(file);
            ((BitmapDrawable) fullPhoto.getDrawable()).getBitmap().compress(Bitmap.CompressFormat.JPEG, 100, out);
            out.flush();
            out.close();

            Toast.makeText(this, getString(R.string.file_downloaded), Toast.LENGTH_SHORT).show();
        } catch (Exception e) {
            Log.e(getClass().getSimpleName(), "Exception downloading the photo: " + e);
        }
    }

    /**
     * @return
     */
    public ImageFetcher getImageFetcher() {
        if (staticImageFetcher == null) {
            ImageCache.ImageCacheParams cacheParams = new ImageCache.ImageCacheParams(this, "thumbs");
            cacheParams.setMemCacheSizePercent(this, 0.5f);

            staticImageFetcher = new ImageFetcher(this, getResources().getDimensionPixelSize(R.dimen.photo_item_size));
            staticImageFetcher.setLoadingImage(R.drawable.thumb_placeholder);
            staticImageFetcher.addImageCache(cacheParams);
        }
        return staticImageFetcher;
    }
}
