/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.activity;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.callback.SearchCallback;
import com.thingspace.cloud.sdk.exception.SearchException;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.FullviewItem;
import com.thingspace.files.R;
import com.thingspace.files.adapters.RecyclerViewGridAdapter;
import com.thingspace.files.listeners.RecyclerViewClickListener;
import com.thingspace.files.views.SpaceItemDecorator;

/**
 * Activity to display Search results
 */
public class SearchResultActivity extends AppCompatActivity implements RecyclerViewClickListener/*implements LoaderManager.LoaderCallbacks<Cursor>, AdapterView.OnItemClickListener*/ {

    public static final String EXTRA_QUERY = "extra_query";
    public static final String EXTRA_QUERY_TYPE = "extra_query_type";
    public static final String EXTRA_SORT_BY = "extra_sortBy";
    public static final String EXTRA_LIMIT = "extra_query_limit";
    public static final String EXTRA_TITLE = "title";

    private static final int COLUMN_COUNT = 3;
    private static final int ACTION_DOWNLOAD = 0;
    protected CloudAPI cloudAPI;
    private RecyclerViewGridAdapter gridCursorAdapter;
    private RecyclerView recyclerView;
    private String searchQuery;
    private String extraQueryType, extraSortBy, extraTitle;
    private int extraLimit = -1;
    private TextView emptyView;
    private ItemList itemListSearchResult;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.resource_recycler_view);

        cloudAPI = new CloudAPI(getString(R.string.APP_ID), getString(R.string.APP_SECRET), getString(R.string.CALLBACK_URL));

        recyclerView = (RecyclerView) findViewById(R.id.recyclerView);
        emptyView = ((TextView) findViewById(R.id.emptyView));
        emptyView.setText(R.string.search_result_empty);
        searchQuery = getIntent().getStringExtra(EXTRA_QUERY);

        GridLayoutManager manager = new GridLayoutManager(this, COLUMN_COUNT);
        manager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                // for thumbs
                return 1;
            }
        });

        recyclerView.setLayoutManager(manager);
        recyclerView.setHasFixedSize(true);
        recyclerView.addItemDecoration(new SpaceItemDecorator(getResources().getDimensionPixelSize(R.dimen.thumb_spacing), 3));

        extraQueryType = getIntent().getStringExtra(EXTRA_QUERY_TYPE);
        extraSortBy = getIntent().getStringExtra(EXTRA_SORT_BY);
        extraLimit = getIntent().getIntExtra(EXTRA_LIMIT, -1);
        extraTitle = getIntent().getStringExtra(EXTRA_TITLE);
        if (!TextUtils.isEmpty(extraTitle)) {
            getSupportActionBar().setTitle(extraTitle);
        }

        Log.e(this.getClass().getSimpleName(), "Search Query: " + searchQuery);

        CloudAPI.QueryType queryType = CloudAPI.QueryType.SQL;
        if (extraQueryType != null && "Lucene".equalsIgnoreCase(extraQueryType)) {
            queryType = CloudAPI.QueryType.LUCENE;
        }

        cloudAPI.search(this, queryType, searchQuery, null, extraSortBy, extraLimit, new SearchCallback() {

            @Override
            public void onError(final SearchException exception) {
                Log.e("SEARCH RESULTS --->", "Search error: " + exception);
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        recyclerView.setVisibility(View.GONE);
                        emptyView.setVisibility(View.VISIBLE);
                        emptyView.setText("Search Exception: " + exception.getMessage());
                    }
                });
            }

            @Override
            public void onSuccess(final ItemList itemList, final boolean isOffline) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Log.e("SEARCH RESULTS --->", "search count: " + itemList.getCount() + " Offline: " + isOffline);
                        if (itemList.getCount() > 0) {
                            emptyView.setVisibility(View.GONE);
                            recyclerView.setVisibility(View.VISIBLE);
                        } else {
                            recyclerView.setVisibility(View.GONE);
                            emptyView.setVisibility(View.VISIBLE);
                            emptyView.setText(R.string.search_result_empty);
                        }

                        emptyView.setVisibility(itemList.getCount() > 0 ? View.GONE : View.VISIBLE);
                        gridCursorAdapter = new RecyclerViewGridAdapter(SearchResultActivity.this, R.layout.grid_item_search, itemList, cloudAPI, SearchResultActivity.this);
                        recyclerView.setAdapter(gridCursorAdapter);
                        itemListSearchResult = itemList;

                        if (isOffline) {
                            Toast.makeText(SearchResultActivity.this, getString(R.string.search_offline_message), Toast.LENGTH_LONG).show();
                        }
                    }
                });
            }
        });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (itemListSearchResult != null) {
            itemListSearchResult.close();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
//        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public void onListItemClicked(View view, int position) {
        FullviewItem fullviewItem = (FullviewItem) itemListSearchResult.getItem(position);
        String mimeType = fullviewItem.getMimeType();
        if (mimeType.indexOf("image") != -1) {
            Intent intent = new Intent(this, PhotoActivity.class);
            intent.putExtra(PhotoActivity.EXTRA_ACCESS_TOKEN, cloudAPI.getAccessToken(SearchResultActivity.this));
            intent.putExtra(PhotoActivity.EXTRA_NAME, fullviewItem.getName());
            intent.putExtra(PhotoActivity.EXTRA_PARENT_PATH, fullviewItem.getParentPath());
            intent.putExtra(PhotoActivity.EXTRA_CAPTURED_DATE, fullviewItem.getCapturedDate());
            intent.putExtra(PhotoActivity.EXTRA_CREATED_DATE, fullviewItem.getVersionCreated());
            intent.putExtra(PhotoActivity.EXTRA_SIZE, String.valueOf(fullviewItem.getSize()));
            intent.putExtra(PhotoActivity.EXTRA_WIDTH, String.valueOf(fullviewItem.getWidth()));
            intent.putExtra(PhotoActivity.EXTRA_HEIGHT, String.valueOf(fullviewItem.getHeight()));
            intent.putExtra(PhotoActivity.EXTRA_MIME_TYPE, fullviewItem.getMimeType());
            intent.putExtra(PhotoActivity.EXTRA_CONTENT_TOKEN, fullviewItem.getContentToken());
            startActivity(intent);
        }
    }

    @Override
    public void onListItemLongClicked(View view, int position) {

    }
}
