/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.activity;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.GridView;
import android.widget.Toast;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.MetadataItem;
import com.thingspace.files.R;
import com.thingspace.files.adapters.MetadataGridAdapter;
import com.thingspace.files.task.DeleteTrashTask;
import com.thingspace.files.task.RestoreTask;
import com.thingspace.files.task.TrashCanTask;

import java.net.HttpURLConnection;
import java.util.ArrayList;

/**
 * Activity to display the Files/Folder View
 */
public class TrashCanActivity extends BaseActivity {

    private static final int ACTION_RESTORE = 0;
    private static String currentFolderOpened = "";
    private ProgressDialog dialog;
    private CloudAPI cloudAPI;
    private GridView gridView;
    private MetadataGridAdapter metaAdapter;
    private ItemList<MetadataItem> metadataResponse;

    private AdapterView.OnItemLongClickListener fileLongClick = new AdapterView.OnItemLongClickListener() {
        @Override
        public boolean onItemLongClick(AdapterView<?> parent, View view, final int position, long id) {
            AlertDialog.Builder trashOperationDialog = new AlertDialog.Builder(TrashCanActivity.this);
            final MetadataItem item = metadataResponse.getItem(position);
            if (item == null) {
                return false;
            }

            final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<String>(
                    TrashCanActivity.this,
                    android.R.layout.simple_list_item_1);
            arrayAdapter.add(getString(R.string.action_restore));


            trashOperationDialog.setAdapter(arrayAdapter, new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int which) {
                    switch (which) {
                        case ACTION_RESTORE:

                            AlertDialog.Builder restoreDialog = new AlertDialog.Builder(TrashCanActivity.this)
                                    .setMessage(getString(R.string.restore_dialog_message))
                                    .setTitle(getString(R.string.action_restore)).setCancelable(false)
                                    .setPositiveButton(getString(R.string.action_yes), new DialogInterface.OnClickListener() {
                                        @Override
                                        public void onClick(DialogInterface dialog, int which) {
                                            showBusyDialog(true, getString(R.string.loading_busy_text));
                                            ArrayList<String> paths = new ArrayList<>();
                                            paths.add(item.getParentPath() + "/" + item.getName());
                                            RestoreTask task = new RestoreTask(TrashCanActivity.this, TrashCanActivity.this, cloudAPI, paths);
                                            task.execute();
                                        }
                                    }).setNegativeButton(getString(R.string.action_no), null);
                            restoreDialog.create();
                            restoreDialog.show();
                            break;
                    }
                }
            });

            trashOperationDialog.setTitle(getString(R.string.file_actions));
            trashOperationDialog.create();
            trashOperationDialog.show();

            return true;
        }
    };

    /**
     * @param savedInstanceState
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        cloudAPI = new CloudAPI(getString(R.string.APP_ID), getString(R.string.APP_SECRET), getString(R.string.CALLBACK_URL));
        gridView = (GridView) findViewById(R.id.gridView);
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (cloudAPI.hasAuthorization(this)) {
            if (currentFolderOpened.isEmpty()) {
                startTrashTask();
            }
        } else {
            startAuthentication();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        currentFolderOpened = "";
    }

    /**
     * To authorize the user.
     */
    private void startAuthentication() {
        // Start the remote authentication
        cloudAPI.authorize(this);
    }

    /**
     * @param menu
     * @return
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_trash_can, menu);
        return true;
    }

    /**
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.

        if (!cloudAPI.hasAuthorization(this)) {
            reStartLogin();
            return super.onOptionsItemSelected(item);
        }

        switch (item.getItemId()) {
            case R.id.action_delete:
                AlertDialog.Builder restoreDialog = new AlertDialog.Builder(TrashCanActivity.this)
                        .setMessage(getString(R.string.trash_purge_dialog_message))
                        .setTitle(getString(R.string.action_delete)).setCancelable(false)
                        .setPositiveButton(getString(R.string.action_yes), new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                showBusyDialog(true, getString(R.string.loading_busy_text));
                                DeleteTrashTask task = new DeleteTrashTask(TrashCanActivity.this, TrashCanActivity.this, cloudAPI);
                                task.execute();
                            }
                        }).setNegativeButton(getString(R.string.action_no), null);
                restoreDialog.create();
                restoreDialog.show();

                break;
        }
        return super.onOptionsItemSelected(item);
    }

    private void startTrashTask() {
        if (cloudAPI.hasAuthorization(this)) {
            showBusyDialog(true, getString(R.string.loading_busy_text));
            TrashCanTask task = new TrashCanTask(this, cloudAPI);
            task.execute();
        } else {
            reStartLogin();
        }
    }


    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
    }

    /**
     * @param response
     */
    @Override
    public void onMetadataResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false, null);

        if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            reStartLogin();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_UNAVAILABLE) {
            Toast.makeText(TrashCanActivity.this, getString(R.string.error_unavailable), Toast.LENGTH_SHORT).show();
            return;
        }

        if (response == null || responseCode != HttpURLConnection.HTTP_OK) {
            Toast.makeText(TrashCanActivity.this, getString(R.string.error_happened), Toast.LENGTH_SHORT).show();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_OK) {
            if (response.getCount() == 0) {
                Toast.makeText(TrashCanActivity.this, getString(R.string.no_files_to_display), Toast.LENGTH_SHORT).show();
                finish();
            }
            this.metadataResponse = response;
            metaAdapter = new MetadataGridAdapter(this, cloudAPI, R.layout.grid_item, response);
            gridView.setAdapter(metaAdapter);
            gridView.setOnItemLongClickListener(fileLongClick);
        }
    }

    @Override
    public void onDeleteResponseReceived(boolean response) {
        showBusyDialog(false, null);
        if (response) {
            startTrashTask();
        }
    }

    /**
     * @param show
     */
    public void showBusyDialog(boolean show, String message) {
        if (dialog == null && show) {
            dialog = new ProgressDialog(TrashCanActivity.this);
            dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            dialog.setMessage(message);
            dialog.setIndeterminate(true);
            dialog.setCanceledOnTouchOutside(false);
            dialog.show();
        } else if (dialog != null && !show) {
            dialog.dismiss();
            dialog = null;

        }
    }

    /**
     * Called to Re-Launch the app when there is a invalid/epired token
     */
    private void reStartLogin() {
        Toast.makeText(TrashCanActivity.this, getString(R.string.error_unathorized), Toast.LENGTH_SHORT).show();
        cloudAPI.logout(this);
        Intent intent = new Intent(TrashCanActivity.this, TrashCanActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivity(intent);
    }

    @Override
    public void onRestoreResponseReceived(Boolean success, int responseCode) {
        showBusyDialog(false, null);
        if (success) {
            startTrashTask();
        }
    }
}