/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.task;

import android.content.Context;
import android.os.AsyncTask;
import android.util.Log;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.exception.CloudAPIException;
import com.thingspace.cloud.sdk.list.item.MetadataItem;
import com.thingspace.files.listeners.FileOperationsResponseListener;

import java.lang.ref.WeakReference;

import javax.net.ssl.HttpsURLConnection;

/**
 * Create folder task.
 */
public class CreateFolderTask extends AsyncTask<String, Void, MetadataItem> {

    private static final String TAG = CreateFolderTask.class.getName();
    private Context context;
    private WeakReference<FileOperationsResponseListener> fileOperationsResponseListener;
    private CloudAPI cloudAPI;
    private String path, folderName;
    private int responseCode;

    /**
     * @param context
     * @param listener
     * @param cloudAPI
     * @param path
     * @param folderName
     */
    public CreateFolderTask(Context context, FileOperationsResponseListener listener, CloudAPI cloudAPI, String path, String folderName) {
        this.context = context;
        fileOperationsResponseListener = new WeakReference<>(listener);
        this.cloudAPI = cloudAPI;
        this.path = path;
        this.folderName = folderName;
    }

    /**
     * @param params
     * @return
     */
    @Override
    protected MetadataItem doInBackground(String... params) {
        try {
            return cloudAPI.createFolder(this.context, path, folderName, null);
        } catch (CloudAPIException ex) {
            Log.e(getClass().getSimpleName(), "Exception retrieving resources: " + ex);
            if (ex.getCode() == CloudAPIException.ErrorCode.USER_UNAUTHAURIZED) {
                responseCode = HttpsURLConnection.HTTP_UNAUTHORIZED;
            } else if(ex.getCode() == CloudAPIException.ErrorCode.ERROR_FILE_ALREADY_EXIST){
                responseCode = HttpsURLConnection.HTTP_OK;
            } else if (ex.getCode() == CloudAPIException.ErrorCode.ERROR_HTTP_CODE) {
                return null;
            }
        }

        return null;
    }

    /**
     * @param metadataItem
     */
    @Override
    protected void onPostExecute(MetadataItem metadataItem) {
        super.onPostExecute(metadataItem);

        if (metadataItem != null) {
            responseCode = HttpsURLConnection.HTTP_CREATED;
            if (metadataItem instanceof MetadataItem) {
                Log.d("------", metadataItem.getName());
            }
        }

        fileOperationsResponseListener.get().onCreateFolderResponseReceived(metadataItem, responseCode);
    }
}