/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.task;

import android.app.Activity;
import android.content.Context;
import android.os.AsyncTask;
import android.util.Log;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.exception.CloudAPIException;
import com.thingspace.files.listeners.FileOperationsResponseListener;

import java.lang.ref.WeakReference;

import javax.net.ssl.HttpsURLConnection;

/**
 * Delete file/folder task.
 */
public class DeleteFileFolderTask extends AsyncTask<String, Void, Boolean> {

    private static final String TAG = DeleteFileFolderTask.class.getName();
    private WeakReference<Activity> activityWeakReference;
    private WeakReference<FileOperationsResponseListener> fileOperationsResponseListener;
    private String path;
    private int responseCode;
    private CloudAPI cloudAPI;
    private Context context;

    /**
     * @param context
     * @param listener
     * @param cloudAPI
     * @param path
     */
    public DeleteFileFolderTask(Context context, FileOperationsResponseListener listener, CloudAPI cloudAPI, String path) {
        this.context = context;
        fileOperationsResponseListener = new WeakReference<>(listener);
        this.cloudAPI = cloudAPI;
        this.path = path;
    }

    /**
     * @param params
     * @return
     */
    @Override
    protected Boolean doInBackground(String... params) {
        try {
            return cloudAPI.delete(this.context, path, false);
        } catch (CloudAPIException ex) {
            Log.e(getClass().getSimpleName(), "Exception retrieving resources: " + ex);
            if (ex.getCode() == CloudAPIException.ErrorCode.USER_UNAUTHAURIZED) {
                responseCode = HttpsURLConnection.HTTP_UNAUTHORIZED;
            } else if (ex.getCode() == CloudAPIException.ErrorCode.ERROR_HTTP_CODE) {
                return false;
            }
        }
        return false;
    }

    /**
     * @param response
     */
    @Override
    protected void onPostExecute(Boolean response) {
        super.onPostExecute(response);
        fileOperationsResponseListener.get().onDeleteResponseReceived(response);
    }
}