/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.task;

import android.app.Activity;
import android.os.AsyncTask;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.exception.CloudAPIException;
import com.thingspace.cloud.sdk.response.AccountInformation;
import com.thingspace.files.listeners.AccountInfoResponseListener;

import java.lang.ref.WeakReference;

import javax.net.ssl.HttpsURLConnection;

/**
 * Fetches the account memory used information.
 */
public class GetAccountInfoTask extends AsyncTask<String, Void, AccountInformation> {

    private static final String TAG = GetAccountInfoTask.class.getName();
    private WeakReference<Activity> activityWeakReference;
    private CloudAPI cloudAPI;
    private int responseCode;
    private WeakReference<AccountInfoResponseListener> accountInfoResponseReference;

    /**
     * @param activity
     * @param cloudAPI
     * @param context
     */
    public GetAccountInfoTask(Activity activity, CloudAPI cloudAPI, AccountInfoResponseListener context) {
        accountInfoResponseReference = new WeakReference<AccountInfoResponseListener>(context);
        activityWeakReference = new WeakReference<>(activity);
        this.cloudAPI = cloudAPI;
    }

    /**
     * @param params
     * @return
     */
    @Override
    protected AccountInformation doInBackground(String... params) {

        if (activityWeakReference.get() == null) {
            throw new IllegalArgumentException("activity reference null");
        }
        try {
            return cloudAPI.getAccountInformation(activityWeakReference.get());
        } catch (CloudAPIException cloudAPIException) {
            if (cloudAPIException.getCode() == CloudAPIException.ErrorCode.USER_UNAUTHAURIZED) {
                this.responseCode = HttpsURLConnection.HTTP_UNAUTHORIZED;
            } else if (cloudAPIException.getCode() == CloudAPIException.ErrorCode.ERROR_LOST_CONNECTION) {
                this.responseCode = HttpsURLConnection.HTTP_UNAVAILABLE;
            }

        }
        return null;
    }


    /**
     * @param accountInformation
     */
    @Override
    protected void onPostExecute(AccountInformation accountInformation) {
        super.onPostExecute(accountInformation);


        if (activityWeakReference.get() == null) {
            throw new IllegalArgumentException("activity reference null");
        }

        if (accountInformation == null) {
            accountInfoResponseReference.get().onAccountInfoResponseReceived(responseCode, 0L, 0L);
            return;
        } else {
            parseAccountInfoResponse(accountInformation);

        }
    }

    /**
     * @param accountInformation
     */
    public void parseAccountInfoResponse(AccountInformation accountInformation) {
        Long availableStorage = 0L, usedStorage = 0L;
        availableStorage = accountInformation.getAvailableStorage();
        usedStorage = accountInformation.getUsedStorage();
        accountInfoResponseReference.get().onAccountInfoResponseReceived(HttpsURLConnection.HTTP_OK, availableStorage, usedStorage);

    }
}
