/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.task;

import android.app.Activity;
import android.os.AsyncTask;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.exception.CloudAPIException;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.files.listeners.MetadataResponseListener;

import java.lang.ref.WeakReference;

import javax.net.ssl.HttpsURLConnection;

/**
 * Fetches Metadata info for the logged in user.
 */
public class MetadataTask extends AsyncTask<String, Void, ItemList> {

    private WeakReference<Activity> activityWeakReference;
    private String path;
    private CloudAPI cloudAPI;
    private int responseCode = -1;

    /**
     * @param activity
     * @param path
     */
    public MetadataTask(Activity activity, String path, CloudAPI cloudAPI) {
        activityWeakReference = new WeakReference<>(activity);
        this.path = path;
        this.cloudAPI = cloudAPI;
    }

    /**
     * @param params
     * @return
     */
    @Override
    protected ItemList doInBackground(String... params) {
        if (activityWeakReference.get() == null) {
            throw new IllegalArgumentException("activity reference null");
        }
        try {
            return cloudAPI.getMetadata(activityWeakReference.get(), path);
        } catch (CloudAPIException cloudAPIException) {
            if (cloudAPIException.getCode() == CloudAPIException.ErrorCode.USER_UNAUTHAURIZED) {
                this.responseCode = HttpsURLConnection.HTTP_UNAUTHORIZED;
            } else if (cloudAPIException.getCode() == CloudAPIException.ErrorCode.ERROR_LOST_CONNECTION) {
                this.responseCode = HttpsURLConnection.HTTP_UNAVAILABLE;
            }
        }
        return null;
    }

    /**
     * @param response
     */
    @Override
    protected void onPostExecute(ItemList response) {
        super.onPostExecute(response);
        if (activityWeakReference.get() == null) {
            throw new IllegalArgumentException("activity reference null");
        }

        if (activityWeakReference.get() instanceof MetadataResponseListener) {
            ((MetadataResponseListener) activityWeakReference.get()).onMetadataResponseReceived(response, response != null ? HttpsURLConnection.HTTP_OK : this.responseCode);
        }
    }
}
