/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.task;

import android.content.Context;
import android.os.AsyncTask;
import android.util.Log;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.exception.CloudAPIException;
import com.thingspace.files.listeners.FileOperationsResponseListener;

import java.lang.ref.WeakReference;
import java.util.ArrayList;

import javax.net.ssl.HttpsURLConnection;

/**
 * Restores deleted entity.
 */
public class RestoreTask extends AsyncTask<String, Void, Boolean> {

    private static final String TAG = RestoreTask.class.getName();
    private Context context;
    private WeakReference<FileOperationsResponseListener> fileOperationsResponseListener;
    private CloudAPI cloudAPI;
    private ArrayList<String> paths = new ArrayList<>();
    private int responseCode;

    /**
     * @param context
     * @param listener
     * @param cloudAPI
     * @param paths
     */
    public RestoreTask(Context context, FileOperationsResponseListener listener, CloudAPI cloudAPI, ArrayList<String> paths) {
        this.context = context;
        fileOperationsResponseListener = new WeakReference<>(listener);
        this.cloudAPI = cloudAPI;
        this.paths = paths;
    }

    /**
     * @param params
     * @return
     */
    @Override
    protected Boolean doInBackground(String... params) {
        try {
            return cloudAPI.restoreTrash(this.context, paths);
        } catch (CloudAPIException ex) {
            Log.e(getClass().getSimpleName(), "Exception restoring resources: " + ex);
            if (ex.getCode() == CloudAPIException.ErrorCode.USER_UNAUTHAURIZED) {
                responseCode = HttpsURLConnection.HTTP_UNAUTHORIZED;
            } else if (ex.getCode() == CloudAPIException.ErrorCode.ERROR_FILE_ALREADY_EXIST) {
                responseCode = HttpsURLConnection.HTTP_OK;
            } else if (ex.getCode() == CloudAPIException.ErrorCode.ERROR_HTTP_CODE) {
                return false;
            }
        }

        return false;
    }

    /**
     * @param success
     */
    @Override
    protected void onPostExecute(Boolean success) {
        super.onPostExecute(success);
        fileOperationsResponseListener.get().onRestoreResponseReceived(success, responseCode);
    }
}