/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.task;

import android.app.Activity;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.Toast;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.exception.CloudAPIException;
import com.thingspace.files.R;
import com.thingspace.files.listeners.FileOperationsResponseListener;

import java.lang.ref.WeakReference;

import javax.net.ssl.HttpsURLConnection;

/**
 * Uploads a file
 */
public class UploadTask extends AsyncTask<String, Void, Boolean> {

    private WeakReference<Activity> activityWeakReference;
    private String uploadPath, fileName, filePath;
    private CloudAPI cloudAPI;
    private int responseCode = -1;

    public UploadTask(Activity activity, String uploadPath, String fileName, String filePath, CloudAPI cloudAPI) {
        activityWeakReference = new WeakReference<>(activity);
        this.uploadPath = uploadPath;
        this.fileName = fileName;
        this.cloudAPI = cloudAPI;
        this.filePath = filePath;
    }

    @Override
    protected Boolean doInBackground(String... params) {
        if (activityWeakReference.get() == null) {
            throw new IllegalArgumentException("activity reference null");
        }

        try {
            return cloudAPI.uploadFile(activityWeakReference.get(), uploadPath, fileName, filePath);
        } catch (CloudAPIException cloudAPIException) {
            if (cloudAPIException.getCode() == CloudAPIException.ErrorCode.USER_UNAUTHAURIZED) {
                this.responseCode = HttpsURLConnection.HTTP_UNAUTHORIZED;
            } else if (cloudAPIException.getCode() == CloudAPIException.ErrorCode.ERROR_FILE_ALREADY_EXIST) {
                this.responseCode = HttpsURLConnection.HTTP_OK;
            } else if (cloudAPIException.getCode() == CloudAPIException.ErrorCode.ERROR_LOST_CONNECTION) {
                this.responseCode = HttpsURLConnection.HTTP_UNAVAILABLE;
            }
        } catch (Exception ex) {
            Log.e(getClass().getSimpleName(), "Exception Uploading file: " + ex);
        }

        return false;
    }

    @Override
    protected void onPostExecute(Boolean result) {
        super.onPostExecute(result);
        if (activityWeakReference.get() == null) {
            throw new IllegalArgumentException("activity reference null");
        }

        ((FileOperationsResponseListener) activityWeakReference.get()).onUploadDownloadResponseReceived(result, this.responseCode);

        if (this.responseCode == HttpsURLConnection.HTTP_UNAUTHORIZED || this.responseCode == HttpsURLConnection.HTTP_UNAVAILABLE) {
            return;
        } else if (this.responseCode == HttpsURLConnection.HTTP_OK) {
            Toast.makeText(activityWeakReference.get(), activityWeakReference.get().getString(R.string.upload_file_exists), Toast.LENGTH_SHORT).show();
            return;
        }

        Toast.makeText(activityWeakReference.get(), activityWeakReference.get().getString(result.booleanValue() ? R.string.file_uploaded : R.string.file_not_uploaded), Toast.LENGTH_SHORT).show();
    }
}
